/*
 *  Copyright 2019-2020 Zheng Jie
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package me.zhengjie.modules.system.service.impl;

import lombok.RequiredArgsConstructor;
import me.zhengjie.config.FileProperties;
import me.zhengjie.domain.LocalStorage;
import me.zhengjie.exception.BadRequestException;
import me.zhengjie.modules.dm.card.domain.DmCard;
import me.zhengjie.modules.dm.card.service.DmCardService;
import me.zhengjie.modules.dm.card.service.dto.DmCardDto;
import me.zhengjie.modules.dm.user.domain.DmUser;
import me.zhengjie.modules.dm.user.repository.DmUserRepository;
import me.zhengjie.modules.dm.user.service.DmUserService;
import me.zhengjie.modules.dm.user.service.dto.DmUserDto;
import me.zhengjie.modules.dm.user.service.mapstruct.DmUserMapper;
import me.zhengjie.modules.security.service.OnlineUserService;
import me.zhengjie.modules.security.service.UserCacheClean;
import me.zhengjie.modules.system.domain.Dept;
import me.zhengjie.modules.system.domain.Job;
import me.zhengjie.modules.system.domain.Role;
import me.zhengjie.modules.system.domain.User;
import me.zhengjie.exception.EntityExistException;
import me.zhengjie.exception.EntityNotFoundException;
import me.zhengjie.modules.system.repository.UserRepository;
import me.zhengjie.modules.system.service.UserService;
import me.zhengjie.modules.system.service.dto.JobSmallDto;
import me.zhengjie.modules.system.service.dto.RoleSmallDto;
import me.zhengjie.modules.system.service.dto.UserDto;
import me.zhengjie.modules.system.service.dto.UserQueryCriteria;
import me.zhengjie.modules.system.service.mapstruct.UserMapper;
import me.zhengjie.repository.LocalStorageRepository;
import me.zhengjie.service.LocalStorageService;
import me.zhengjie.utils.*;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Zheng Jie
 * @date 2018-11-23
 */
@Service
@RequiredArgsConstructor
@CacheConfig(cacheNames = "user")
public class UserServiceImpl implements UserService {

    private final UserRepository userRepository;
    private final UserMapper userMapper;
    private final FileProperties properties;
    private final RedisUtils redisUtils;
    private final UserCacheClean userCacheClean;
    private final OnlineUserService onlineUserService;
    private final DmUserService dmUserService;
    private final DmUserMapper dmUserMapper;
    private final DmUserRepository dmUserRepository;
    private final DmCardService dmCardService;
    @Override
    public Object queryAll(UserQueryCriteria criteria, Pageable pageable) {
        Page<User> page = userRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root, criteria, criteriaBuilder), pageable);
        return PageUtil.toPage(page.map(userMapper::toDto));
    }

    @Override
    public List<UserDto> queryAll(UserQueryCriteria criteria) {
        List<User> users = userRepository.findAll((root, criteriaQuery, criteriaBuilder) -> QueryHelp.getPredicate(root, criteria, criteriaBuilder));
        return userMapper.toDto(users);
    }

    @Override
    @Cacheable(key = "'id:' + #p0")
    @Transactional(rollbackFor = Exception.class)
    public UserDto findById(String id) {
        Optional<User> user = userRepository.findById(id);
        if (!user.isPresent()) {
            return null;
        } else {
            ValidationUtil.isNull(user.get().getId(), "User", "id", id);
            return userMapper.toDto(user.get());
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void create(User resources) {
        //设置用户信息
        //resources = setDmUserInfoToSysUser(resources);
        if (userRepository.findByUsername(resources.getUsername()) != null) {
            throw new EntityExistException(User.class, "username", resources.getUsername());
        }
        resources.setId(UUID.randomUUID().toString());

        DmUser dmUser = new DmUser();
        dmUser.setId(resources.getId());
        dmUser.setName(resources.getNickName());
        dmUser.setSex(resources.getGender());
        dmUser.setContacts(resources.getPhone());
        dmUser.setEmail(resources.getEmail());
        dmUser.setDept(resources.getDept());
        dmUser.setUserNumber(resources.getUserCode());
        dmUser.setUserCode(resources.getUserCode());
        dmUser.setStatus(resources.getEnabled());
        dmUser.setType("01");
        DmCardDto dmCardDto = dmCardService.create(new DmCard());
        dmUser.setCardid(String.valueOf(dmCardDto.getId()));
        DmUser dmUser1 = dmUserRepository.save(dmUser);

        resources.setDmCompany(null);
        resources.setDmUser(dmUser1);
        userRepository.save(resources);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(User resources) throws Exception {
        User user = userRepository.findById(resources.getId()).orElseGet(User::new);
        User user1 = userRepository.findByUsername(resources.getUsername());
        if (user1 != null && !user.getId().equals(user1.getId())) {
            throw new EntityExistException(User.class, "username", resources.getUsername());
        }
        // 如果用户的角色改变
        if (!resources.getRoles().equals(user.getRoles())) {
            redisUtils.del(CacheKey.DATA_USER + resources.getId());
            redisUtils.del(CacheKey.MENU_USER + resources.getId());
            redisUtils.del(CacheKey.ROLE_AUTH + resources.getId());
        }

        // 如果用户被禁用，则清除用户登录信息
        if(!resources.getEnabled()){
            onlineUserService.kickOutForUsername(resources.getUsername());
        }

        DmUser dmUser = dmUserRepository.findById(resources.getId()).orElseGet(DmUser::new);
        //DmUser dmUser = new DmUser();
        dmUser.setId(resources.getId());
        dmUser.setName(resources.getNickName());
        dmUser.setSex(resources.getGender());
        dmUser.setContacts(resources.getPhone());
        dmUser.setEmail(resources.getEmail());
        dmUser.setDept(resources.getDept());
        dmUser.setUserNumber(resources.getUserCode());
        dmUser.setUserCode(resources.getUserCode());
        dmUser.setStatus(resources.getEnabled());
        dmUser.setType("01");
        DmUser dmUser1 = dmUserRepository.save(dmUser);

        resources.setDmCompany(null);
        resources.setDmUser(dmUser1);
        userRepository.save(resources);
        // 清除缓存
        delCaches(resources.getId(), resources.getUsername());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createSync(User resources) {
        //设置用户信息
        resources = setDmUserInfoToSysUser(resources);
        if (userRepository.findByUsername(resources.getUsername()) != null) {
            throw new EntityExistException(User.class, "username", resources.getUsername());
        }
        /*if (userRepository.findByEmail(resources.getEmail()) != null) {
            throw new EntityExistException(User.class, "email", resources.getEmail());
        }*/
//        if (userRepository.findByPhone(resources.getPhone()) != null) {
//            throw new EntityExistException(User.class, "phone", resources.getPhone());
//        }
        userRepository.save(resources);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateSync(User resources) throws Exception {
        User user = userRepository.findById(resources.getId()).orElseGet(User::new);
        ValidationUtil.isNull(user.getId(), "User", "id", resources.getId());
        // 如果用户被禁用，则清除用户登录信息
        if(!resources.getEnabled()){
            onlineUserService.kickOutForUsername(resources.getUsername());
        }
        user.setUsername(resources.getUsername());
        user.setEmail(resources.getEmail());
        user.setEnabled(resources.getEnabled());
        user.setRoles(resources.getRoles());
        user.setDept(resources.getDept());
        user.setJobs(resources.getJobs());
        user.setPhone(resources.getPhone());
        user.setNickName(resources.getNickName());
        user.setGender(resources.getGender());
        user.setRemark(resources.getRemark());
        user.setErrLimit(resources.getErrLimit());
        user.setDmUser(resources.getDmUser());
        user.setDmCompany(resources.getDmCompany());
        user.setUserRootGroupId(resources.getUserRootGroupId());
        user.setUserRootGroupName(resources.getUserRootGroupName());
        user.setUserCode(resources.getUserCode());
        if(StringUtils.isBlank(user.getOrgPassword())){
            user.setOrgPassword(resources.getOrgPassword());
        }
        userRepository.save(user);
        // 清除缓存
        delCaches(user.getId(), user.getUsername());
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateCenter(User resources) {
        User user = userRepository.findById(resources.getId()).orElseGet(User::new);
        User user1 = userRepository.findByPhone(resources.getPhone());
        if (user1 != null && !user.getId().equals(user1.getId())) {
            throw new EntityExistException(User.class, "phone", resources.getPhone());
        }
        user.setNickName(resources.getNickName());
        user.setPhone(resources.getPhone());
        user.setGender(resources.getGender());
        userRepository.save(user);
        // 清理缓存
        delCaches(user.getId(), user.getUsername());
    }

    @Override
    public List<User> findByDmUserId(String dmUserId,String userId) {
        return userRepository.findByDmUserId(dmUserId,userId);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Set<String> ids) {
        for (String id : ids) {
            // 清理缓存
            UserDto user = findById(id);
            delCaches(user.getId(), user.getUsername());
            userRepository.deleteUserJobs(id);
            userRepository.deleteUserRoles(id);
            dmUserRepository.deleteById(id);
            userRepository.deleteById(id);
        }
    }

    @Override
    public UserDto findByName(String userName) {
        User user = userRepository.findByUsername(userName);
        if (user == null) {
            throw new EntityNotFoundException(User.class, "name", userName);
        } else {
            return userMapper.toDto(user);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePass(String username, String pass) {
        userRepository.updatePass(username, pass, new Date());
        flushCache(username);
    }

    @Override
    public void updatePassById(String userId, String encryptPassword) {
        userRepository.updatePassById(userId, encryptPassword, new Date());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Map<String, String> updateAvatar(MultipartFile multipartFile) {
        //根据用户名查询用户  一对一  精准查询
        User user = userRepository.findByUsername(SecurityUtils.getCurrentUsername());
        //获取头像旧路径
        String oldPath = user.getAvatarPath();
        //头像图片上传
        File file = FileUtil.upload(multipartFile, properties.getPath().getAvatar());
        //设置头像路径
        user.setAvatarPath(Objects.requireNonNull(file).getPath());
        //设置头像名称
        user.setAvatarName(file.getName());
        //保存用户  会根据数据自动修改
        userRepository.save(user);
        //判断旧文件地址是否是null的
        if (StringUtils.isNotBlank(oldPath)) {
            //根据旧文件地址删除文件
            if (FileUtil.del(oldPath)){
                //删除成功需要删除对应文件表中的数据 tool_local_storage
//                this.localStorageService.deleteByUrl(oldPath);
            }
        }
        //获取用户名称不能为NULL
        @NotBlank String username = user.getUsername();
        //清理用户登录时的缓存
        flushCache(username);
        return new HashMap<String, String>(1) {{
            put("avatar", file.getName());
        }};
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateEmail(String username, String email) {
        userRepository.updateEmail(username, email);
        flushCache(username);
    }

    @Override
    public void download(List<UserDto> queryAll, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> list = new ArrayList<>();
        for (UserDto userDTO : queryAll) {
            List<String> roles = userDTO.getRoles().stream().map(RoleSmallDto::getName).collect(Collectors.toList());
            Map<String, Object> map = new LinkedHashMap<>();
            map.put("用户名", userDTO.getUsername());
            map.put("角色", roles);
            map.put("部门", userDTO.getDept().getName());
            map.put("岗位", userDTO.getJobs().stream().map(JobSmallDto::getName).collect(Collectors.toList()));
            map.put("邮箱", userDTO.getEmail());
            map.put("状态", userDTO.getEnabled() ? "启用" : "禁用");
            map.put("手机号码", userDTO.getPhone());
            map.put("修改密码的时间", userDTO.getPwdResetTime());
            map.put("创建日期", userDTO.getCreateTime());
            list.add(map);
        }
        FileUtil.downloadExcel(list, response);
    }

    /**
     * 清理缓存
     *
     * @param id /
     */
    public void delCaches(String id, String username) {
        redisUtils.del(CacheKey.USER_ID + id);
        flushCache(username);
    }

    /**
     * 清理 登陆时 用户缓存信息
     *
     * @param username /
     */
    private void flushCache(String username) {
        userCacheClean.cleanUserCache(username);
    }

    /**
     * 将绑定的DM用户信息跟账号进行关联
     * @param resources
     */
    public User setDmUserInfoToSysUser(User resources){
        //判断关联的DM用户是否已关联别的账号
        if(resources != null && resources.getDmUser() != null && resources.getDmUser().getId() != null){
            List<User> list = findByDmUserId(resources.getDmUser().getId(),resources.getId());
            if(list != null && list.size() > 0){
                throw new BadRequestException("该用户已绑定账号,请重新选择绑定用户");
            }
        }
        if(resources.getDmUser() != null && resources.getDmUser().getId() != null){
            //查询关联的DM用户信息
            DmUserDto dmUserDto = dmUserService.findById(resources.getDmUser().getId());
            if(dmUserDto != null){
                List<DmUserDto> temp = new ArrayList<>();
                temp.add(dmUserDto);
                DmUser dmUser = dmUserMapper.toEntity(temp).get(0);
                //设置对应的信息
                resources.setNickName(dmUser.getName());//昵称/用户名
                resources.setGender(dmUser.getSex());//性别
                resources.setEmail(dmUser.getEmail());//邮箱
                resources.setExtendsCol(resources.getOrgPassword()+"||"+resources.getPassword());//现密码
                resources.setDept(dmUser.getDept());//部门
                resources.setJobs(dmUser.getJobs());//岗位
                resources.setRoles(dmUser.getRoles());//角色
                //判断是否是管理员
                for(Role r : dmUser.getRoles()){
                    if(StringUtils.isNotBlank(r.getRoleKey()) && r.getRoleKey().equals("admin")){
                        resources.setIsAdmin(true);
                        break;
                    }
                }
            }
        }

        return resources;
    }

    @Override
    public String findPassWord(String username) {
        return userRepository.findPassWord(username);
    }

    /**
     * 解绑用户
     * @param ids
     */
    public void unBindDmUser(Set<String> ids) throws Exception{
        for(String id: ids){
            //查询用户信息
           User user = userMapper.toEntity(findById(id));
           if(user != null){
               user.setIsAdmin(false);
               user.setDmUser(null);
               user.setEmail(null);
               user.setGender(null);
               user.setNickName(null);
               user.setRoles(new HashSet<>());
               user.setJobs(new HashSet<>());
               user.setDept(null);
               update(user);
           }
        }
    }
}
